(function() {
  var Column, Row, Table, Util;
  Util = require('./util');
  Column = require('./column');
  Row = require('./row');
  Table = (function() {
    var autoSeparate, autoSeparateAll, autoSeparateFooter, autoSeparateHeader, autoSeparateNone, corner, defaultColumnAlignment, horizontal, padding, repeat, rows, vertical;
    autoSeparateNone = 0x0;
    autoSeparateHeader = 0x1;
    autoSeparateFooter = 0x2;
    autoSeparateAll = 0x3;
    corner = '+';
    horizontal = '-';
    vertical = '|';
    padding = 0;
    defaultColumnAlignment = [];
    rows = [];
    autoSeparate = autoSeparateAll;
    repeat = Util.repeat;
    function Table(columnWidths) {
      this.columnWidths = columnWidths;
    }
    Table.prototype.setDefaultColumnAlignment = function(alignment) {
      defaultColumnAlignment = alignment;
      return this;
    };
    Table.prototype.render = function() {
      var columnNum, columnWidth, columnWidths, connector, currentLowerColumn, currentLowerWidth, currentUpperColumn, currentUpperWidth, drawSeparator, lastColumnWidths, numColumns, numRows, renderedRow, result, row, rowNum, _len, _len2, _ref;
      if (rows.length === 0) {
        throw new Error('No rows added to the table yet');
      }
      result = '';
      numRows = rows.length;
      for (rowNum = 0, _len = rows.length; rowNum < _len; rowNum++) {
        row = rows[rowNum];
        if (columnWidths) {
          lastColumnWidths = columnWidths;
        }
        renderedRow = row.render(this.columnWidths, padding);
        columnWidths = row.getColumnWidths();
        numColumns = columnWidths.length;
        if (rowNum === 0) {
          result = this.drawBorder(result, columnWidths);
        } else {
          if (autoSeparate & autoSeparateAll) {
            drawSeparator = true;
          } else if (rowNum === 1 && autoSeparate & autoSeparateHeader) {
            drawSeparator = true;
          } else if (rowNum === (numRows - 1) && autoSeparate & autoSeparateFooter) {
            drawSeparator = true;
          } else {
            drawSeparator = false;
          }
          if (drawSeparator) {
            result += vertical;
            currentUpperColumn = 0;
            currentLowerColumn = 0;
            currentUpperWidth = 0;
            currentLowerWidth = 0;
            _ref = this.columnWidths;
            for (columnNum = 0, _len2 = _ref.length; columnNum < _len2; columnNum++) {
              columnWidth = _ref[columnNum];
              result += repeat(horizontal, columnWidth);
              if (columnNum + 1 === this.columnWidths.length) {
                break;
              }
              connector = 0x0;
              currentUpperWidth += columnWidth;
              currentLowerWidth += columnWidth;
              if (lastColumnWidths[currentUpperColumn] === currentUpperWidth) {
                connector |= 0x1;
                currentUpperColumn += 1;
                currentUpperWidth = 0;
              } else {
                currentUpperWidth += 1;
              }
              if (columnWidths[currentLowerColumn] === currentLowerWidth) {
                connector |= 0x2;
                currentLowerColumn += 1;
                currentLowerWidth = 0;
              } else {
                currentLowerWidth += 1;
              }
              switch (connector) {
                case 0x0:
                  result += horizontal;
                  break;
                default:
                  result += corner;
              }
            }
            result += vertical + "\n";
          }
        }
        result += renderedRow;
        if (rowNum + 1 === numRows) {
          result = this.drawBorder(result, columnWidths);
        }
      }
      return result;
    };
    Table.prototype.appendRow = function(row) {
      var align, data, i, item, _len;
      if (row.length > this.columnWidths.length) {
        throw new Error('The row contains too many columns');
      }
      data = row;
      row = new Row;
      for (i = 0, _len = data.length; i < _len; i++) {
        item = data[i];
        align = defaultColumnAlignment[i] || null;
        row.appendColumn(new Column(item, align));
      }
      rows[rows.length] = row;
      return this;
    };
    Table.prototype.drawBorder = function(result, columnWidths) {
      var columnWidth, _i, _len;
      result += corner;
      for (_i = 0, _len = columnWidths.length; _i < _len; _i++) {
        columnWidth = columnWidths[_i];
        result += repeat(horizontal, columnWidth) + corner;
      }
      return result += "\n";
    };
    return Table;
  })();
  module.exports = Table;
}).call(this);
